/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code.  The copyright
notice included with the software must be maintained in all copies
produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1994-2008, John Conover, All Rights Reserved.

This program is based on temp.c, (from the the Maxim/Dallas
Semiconductor SDK kit, uLinuxGNU310r2.zip, at
http://www.maxim-ic.com/products/ibutton/software/1wire/wirekit.cfm,)
which contained the following license:

    Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
    OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
    ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
    OTHER DEALINGS IN THE SOFTWARE.

    Except as contained in this notice, the name of Dallas Semiconductor
    shall not be used except as stated in the Dallas Semiconductor
    Branding Policy.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

    http://www.johncon.com/

------------------------------------------------------------------------------

tempS0.c is for interrogating temperature transducers, (like the
Maxim/Dallas Semiconductor DS08S20 1-Wire Digital Thermometer,) on a
1-wire network through a Maxim/Dallas Semiconductor DS9097U Universal
COM Port Adaptor, (or equivalent,) serial port connection.

The default port is /dev/ttyS0. The default semaphore lock file is
/var/lock/LCK..ttyS0, which has an fcntl(2) advisory lock applied.

As an example usage:

    tempS0 -s 360 -o /var/log/temperature/temperature.log \
           -n AA0008014C02A810=John -n 670008014C733C10=DataC \
           -n 520008014BEBD210=Attic -n 7200080194479610=Ambient \
           -n 3700080193FE8910=Den

would log the temperature in the file temperature.log on the 6 minute
marker from the system clock, (i.e., at 0, 6, 12, 18 ... minutes after
the hour,) for the 5 devices, using the device aliases in the log
file; the log file is a tab delimited flat database text file,
collated in temporal order.

$Revision: 0.0$
$Date: 2008/09/07 21:31:06 $
$Id: tempS0.c,v 0.0 2008/09/07 21:31:06 john Exp $
$Log: tempS0.c,v $
Revision 0.0  2008/09/07 21:31:06  john
Initial version

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <sys/stat.h>

#include "ownet.h"
#include "temp10.h"
#include "findtype.h"

#define MAXDEVICES 20 /* maximum number of thermometer devices */
#define BUFLEN BUFSIZ /* size of record buffers */
#define SPDEVICE "/dev/ttyS0" /* default serial port device */
#define LOCKFILE "/var/lock/LCK..ttyS0" /* default serial port lock file name */

static char rcsid[] = "$Id: tempS0.c,v 0.0 2008/09/07 21:31:06 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1994-2009, John Conover, All Rights Reserved"; /* the copyright banner */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "1-Wire network temperature communication\n",
    "Usage: tempS0 [-d device] [-i] [-l lockfile] [-n serial_number=alias]\n",
    "              [-o outfile] [-s seconds]\n",
    "    -d device, serial device name, (/dev/ttyS0)\n",
    "    -i, scan the 1-wire port for device serial numbers\n",
    "    -l lockfile, lock file name, (/var/lock/LCK..ttyS0)\n",
    "    -n serial_number=alias, device serial number(s) and alias(es), one per device\n",
    "    -o outfile, output file name, (stdout)\n",
    "    -s seconds, time interval to wait between temperature readings, (0)\n",
    "    -v, print the program's version information\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error creating lock file\n",
    "Error locking lock file\n",
    "Error writing to lock file\n",
    "Error installing signal handler\n",
    "Error opening 1-wire network\n",
    "Error getting time of day\n",
    "Error opening output file\n",
    "Error writing to output file\n",
    "Error closing output file\n"
};

#define NOERROR 0 /* negative error values, one for each index in the error message array */
#define EARGS 1
#define ELOCKF 2
#define EFCNL 3
#define EWLCK 4
#define ESIG 5
#define ENET 6
#define ETIME 7
#define EOPEN 8
#define EWRITE 9
#define ECLOSE 10

typedef struct device_struct /* device structure, one per thermometer device */
{
    uchar SN[8]; /* device serial number */
    char *HSN; /* reference to device hex serial number */
    char *alias; /* reference to device alias */
} DEVICE;

static char serial_lockname[] = {LOCKFILE}, /* default lock file name */
            *lockname = serial_lockname, /* default lock file name */
            serial_device[] = {SPDEVICE}, /* default serial port device name */
            *outfile_name = (char *) 0; /* reference to output file name */

static int semaphorelock = -1, /* lock file file descriptor */
           fcntl_lock = -1, /* return value of locking the lock file */
           portnum = -1; /* 1-wire network port number */

static struct flock lockfile; /* lock file structure */

#ifdef __STDC__

static void print_message (int pretval); /* print any error messages */
static void spclose (int err); /* interrupt handler */

#else

static void print_message (); /* print any error messages */
static void spclose (); /* interrupt handler */

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char c, /* command line switch */
         *device = serial_device, /* default serial port device name */
         hex_str[5], /* 2 character hexadecimal string, with leading "0x" and trailing '\0' */
         pid_str[BUFLEN]; /* pid string */

    uchar SN[MAXDEVICES][8]; /* device serial numbers */

    int retval = NOERROR, /* return value, assume no error */
        s = 0, /* time interval to wait between temperature readings in seconds */
        scan_flag = 0, /* scan for device serial numbers flag, 0 = no, 1 = yes */
        device_count = 0, /* count of thermometer devices */
        hex_char_ctr, /* hexadecimal string character counter */
        SN_byte_ctr, /* serial number byte counter */
        i; /* general counter */

    size_t len; /* string length of hexadecimal serial number */

    float current_temp; /* current temperature of a thermometer device */

    FILE *outfile = stdout; /* reference to output file */

    pid_t pid_program = (pid_t) 0; /* pid of parent write process */

    time_t timestamp; /* current time */

    struct tm *time_stamp; /* current time structure */

    DEVICE devices[MAXDEVICES]; /* array of device structures */

    pid_program = getpid (); /* save the pid of parent write process */

    lockfile.l_type = F_WRLCK; /* exclusive write lock the entire file */
    lockfile.l_whence = (short int) SEEK_SET;
    lockfile.l_start = (off_t) 0;
    lockfile.l_len = (off_t) 0;

    hex_str[0] = '0'; /* initialize the 2 character hexadecimal string, with leading "0x" and trailing '\0' */
    hex_str[1] = 'x';
    hex_str[4] = '\0';

    while ((c = getopt (argc, argv, "d:il:n:o:s:v")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'd': /* request for device name? */

                device = optarg; /* yes, save the device name */
                break;

            case 'i': /* scan for device serial numbers, 0 = no, 1 = yes? */

                scan_flag = 1; /* set the scan for device serial numbers flag, 0 = no, 1 = yes */
                break;

            case 'l': /* request for lock file name? */

                lockname = optarg; /* yes, save the lock file name */
                break;

            case 'n': /* request for device serial number and alias? */

                if ((len = strspn (optarg, "0123456789abcdefABCDEF")) != 16) /* device serial number must be exactly 16 hex ascii characters */
                {
                    optind = argc; /* force argument error */
                    retval = EARGS; /* assume not enough arguments */
                }

                else
                {
                    devices[device_count].HSN = optarg; /* save the reference to device hex serial number */

                    if (optarg[16] == '\0') /* no alias? */
                    {
                        devices[device_count].alias = (char *) 0; /* no alias, null reference */
                    }

                    else
                    {
                        devices[device_count].alias = &optarg[17]; /* save the reference to the device alias */
                    }

                    /* the following is the inverse of void PrintSerialNum (uchar* buffer) in ioutil.c */

                    for (hex_char_ctr = 0, SN_byte_ctr = 7; hex_char_ctr <= 14; hex_char_ctr = hex_char_ctr + 2, SN_byte_ctr --) /* for every other character in the device hexadecimal serial number */
                    {
                        hex_str[2] = optarg[hex_char_ctr]; /* first hex character */
                        hex_str[3] = optarg[hex_char_ctr + 1]; /* second hex character */
                        devices[device_count].SN[SN_byte_ctr] = (uchar) strtol (hex_str, (char **) 0, 0); /* save the device serial number byte */
                    }

                    device_count ++; /* increment the count of thermometer devices */
                }

                break;

            case 'o': /* request for output file name? */

                outfile_name = optarg; /* yes, save the output file name */
                break;

            case 's': /* request for time interval to wait between temperature readings in seconds? */

                s = atoi (optarg); /* yes, save the time interval to wait between temperature readings in seconds */

                if (s < 0) /* sanity check, s = 0 means go as fast as possible */
                {
                    optind = argc; /* force argument error */
                    retval = EARGS; /* assume not enough arguments */
                }

                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {
        retval = ESIG; /* assume error installing signal handler */

        if (signal (SIGINT, spclose) != SIG_ERR) /* install the interrupt handler for SIGINT = ^C */
        {

            if (signal (SIGTERM, spclose) != SIG_ERR) /* install the interrupt handler for SIGTERM */
            {

                if (signal (SIGCHLD, spclose) != SIG_ERR) /* install the interrupt handler for SIGCHLD */
                {
                    retval = ELOCKF; /* assume error creating lock file */

                    if ((semaphorelock = open (lockname, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH)) >= 0) /* open the lock file exclusively */
                    {
                        retval = NOERROR; /* assume no error */
                    }

                    else if ((semaphorelock = open (lockname, O_RDWR, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH)) >= 0) /* open the lock file if it exists, it may be a stale lock file, or some other program may own it-in which case a fcntl () lock will be on the file */
                    {
                        retval = NOERROR; /* assume no error */
                    }

                    if (retval == NOERROR) /* any errors? */
                    {
                        lockfile.l_type = F_WRLCK; /* exclusive write lock the entire file */
                        retval = EFCNL; /* assume error locking lock file */

                        if ((fcntl_lock = fcntl (semaphorelock, F_SETLK, &lockfile)) == 0) /* exclusive write lock the lock file */
                        {
                            retval = EWLCK; /* assume error writing to lock file */

                            if (ftruncate (semaphorelock, 0) == 0) /* truncate the lock file */
                            {
                                (void) sprintf (pid_str, "%10d\n", pid_program); /* the pid string is the PID right justified to 10 characters plus a '\n' EOL */

                                if (write (semaphorelock, pid_str, strlen (pid_str)) >= 0) /* write the serial port dialer string to the serial port */
                                {
                                    retval = ENET; /* assume error opening 1-wire network */

                                    if ((portnum = owAcquireEx (device)) >= 0) /* open the 1-wire network */
                                    {
                                        retval = NOERROR; /* assume no error */

                                        if (scan_flag == 1) /* scan for device serial numbers flag, 0 = no, 1 = yes? */
                                        {

                                            if ((device_count = FindDevices (portnum, &SN[0], 0x10, MAXDEVICES)) > 0) /* find the devices on the 1-wire network */
                                            {

                                                for (i = 0; i < device_count; i ++) /* for each device found */
                                                {
                                                    PrintSerialNum (SN[i]); /* print the device's hexadecimal serial number */
                                                    (void) printf ("\n");
                                                }

                                            }

                                        }

                                        else
                                        {

                                            if (device_count == 0) /* any devices specified on the command line? */
                                            {

                                                if ((device_count = FindDevices (portnum, &SN[0], 0x10, MAXDEVICES)) > 0) /* no, find the devices on the 1-wire network */
                                                {

                                                    for (i = 0; i < device_count; i ++) /* for each device found */
                                                    {
                                                        memcpy (devices[i].SN, SN[i], 8); /* save the device's serial number */
                                                        devices[i].HSN = (char *) 0; /* no device hex serial number */
                                                        devices[i].alias = (char *) 0; /* no device alias */
                                                    }

                                                    while (retval == NOERROR) /* while forever, waiting for an interrupt to vector through spclose () */
                                                    {
                                                        retval = ETIME; /* assume error getting time of day */

                                                        if ((timestamp = time ((time_t *) 0)) >= (time_t) 0) /* get the time of day */
                                                        {

                                                            if (s > 0) /* s = 0 means as fast as possible */
                                                            {
                                                                (void) sleep (((timestamp / s) * s) + s - timestamp); /* wait for the next system time marker */

                                                            }

                                                            retval = NOERROR; /* assume no error */

                                                            for (i = 0; i < device_count; i++) /* for each device found */
                                                            {
                                                                retval = ETIME; /* assume error getting time of day */

                                                                if ((timestamp = time ((time_t *) 0)) >= (time_t) 0) /* get the time of day */
                                                                {

                                                                    if ((time_stamp = localtime (&timestamp)) != (struct tm *) 0)
                                                                    {
                                                                        retval = NOERROR; /* assume no error */

                                                                        if (ReadTemperature (portnum, devices[i].SN, &current_temp)) /* get the device's temperature */
                                                                        {
                                                                            (void) printf ("%d/%02d/%02d\t%02d:%02d:%02d\t%d\t", time_stamp->tm_year + 1900, time_stamp->tm_mon + 1, time_stamp->tm_mday, time_stamp->tm_hour, time_stamp->tm_min, time_stamp->tm_sec, (int) timestamp); /* print the time stamp */
                                                                            PrintSerialNum (devices[i].SN); /* print the device's hex serial number */
                                                                            (void) printf ("\t%5.2f \n", (current_temp * ((double) 9.0 / (double) 5.0)) + (double) 32.0); /* and print the devices temperature */
                                                                        }

                                                                        else
                                                                        {
                                                                            (void) fprintf (stderr, "Error reading temperature, device verification = %d\n", (int) owVerify (portnum, FALSE)); /* couldn't get the device's temperature, print why */
                                                                        }

                                                                    }

                                                                }

                                                            }

                                                        }

                                                    }

                                                }

                                            }

                                            else
                                            {

                                                while (retval == NOERROR) /* while forever, waiting for an interrupt to vector through spclose () */
                                                {
                                                    retval = ETIME; /* assume error getting time of day */

                                                    if ((timestamp = time ((time_t *) 0)) >= (time_t) 0) /* get the time of day */
                                                    {

                                                        if (s > 0) /* s = 0 means as fast as possible */
                                                        {
                                                            (void) sleep (((timestamp / s) * s) + s - timestamp); /* wait for the next system time marker */

                                                        }

                                                        retval = EOPEN; /* assume error opening output file */

                                                        if ((outfile = outfile_name == (char *) 0 ? stdout : fopen (outfile_name, "a")) != (FILE *) 0)
                                                        {

                                                            for (i = 0; i < device_count; i++) /* for each device found */
                                                            {
                                                                retval = ETIME; /* assume error getting time of day */

                                                                if ((timestamp = time ((time_t *) 0)) >= (time_t) 0) /* get the time of day */
                                                                {

                                                                    if ((time_stamp = localtime (&timestamp)) != (struct tm *) 0)
                                                                    {

                                                                        if (ReadTemperature (portnum, devices[i].SN, &current_temp)) /* get the device's temperature */
                                                                        {
                                                                            retval = EWRITE; /* assume error writing to output file */

                                                                            if (fprintf (outfile, "%d/%02d/%02d\t%02d:%02d:%02d\t%d\t",     time_stamp->tm_year + 1900, time_stamp->tm_mon + 1, time_stamp->tm_mday, time_stamp->tm_hour, time_stamp->tm_min, time_stamp->tm_sec, (int) timestamp) >= 0) /* print the time stamp */
                                                                            {

                                                                                if (fprintf (outfile, "%s", devices[i].alias) >= 0) /* print the device's alias */
                                                                                {

                                                                                    if (fprintf (outfile, "\t%5.2f\n", (current_temp * ((double) 9.0 / (double) 5.0)) + (double) 32.0) >= 0) /* and print the devices temperature */
                                                                                    {
                                                                                        retval = NOERROR; /* assume no error */
                                                                                    }

                                                                                }

                                                                            }

                                                                        }

                                                                        else
                                                                        {
                                                                            (void) fprintf (stderr, "Error reading temperature, device verification = %d\n", (int) owVerify (portnum, FALSE)); /* couldn't get the device's temperature, print why */
                                                                        }

                                                                    }

                                                                }

                                                            }

                                                            if (outfile != stdout) /* output file open? */
                                                            {
                                                                retval = ECLOSE; /* assume error closing output file */

                                                                if (fflush (outfile) == 0) /* flush the output file */
                                                                {

                                                                    if (fclose (outfile) == 0) /* close the output file */
                                                                    {
                                                                        retval = NOERROR; /* assume no error */
                                                                    }

                                                                }

                                                            }

                                                        }

                                                    }

                                                }

                                            }

                                        }

                                    }

                                }

                            }

                        }

                    }

                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    spclose (retval); /* shutdown */
    exit (retval); /* exit with the error value, formality: spclose () did the exit */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}

/*

Interrupt handler and shutdown.

static void spclose (int err);

Close, unlock, and remove the lock file, and restore the signal
handlers.

There is no return from this routine. The routine exits with a value
of zero if no errors or interrupts occurred, or, the value of the
interrupt, (a positive number,) if an interrupt occurred, or, with a
negative error number if an an error occurred.

*/

#ifdef __STDC__

static void spclose (int err)

#else

static void spclose (err)
int err;

#endif

{

    if (portnum >= 0) /* 1-wire network open? */
    {
        owRelease (portnum); /* close the 1-wire network */
    }

    if (fcntl_lock == 0) /* return value of locking the lock file by fcntl () indicate a successful lock? */
    {

        /*

        the rule is that whichever process owns the exclusive fcntl ()
        lock on the lock file owns the lock file, (and the 1-wire
        port, which has just been closed,) and only that process can
        remove the lock file

        */

        lockfile.l_type = F_UNLCK; /* unlock the file */
        (void) fcntl (semaphorelock, F_SETLKW, &lockfile); /* unlock the lock file */
        (void) unlink (lockname); /* this process owns the lock file, remove the lock file */

        /*

        note that the lock file may not have been removed-another
        process may have the lock file open, waiting on the fcntl ()
        lock to be removed by this process, in which case it will use
        the current lock file; it does not matter since this process
        has already closed the 1-wire port, and will close the lock
        file in the following exit () statement

        */

    }

    /*

    for formality, the output file may be open, but would be closed by
    exit ()

    for formality, the signal () vector on SIGCHLD, SIGTERM, and,
    SIGINT, will be restored by exit ()

    */

    exit (err); /* exit with the value of the interrupt, the value of an error in the main body of this program, the value of an error in this routine, or zero for no error */
}
